@@ -40,4 +40,7 @@ void grd_rdp_server_stop (GrdRdpServer *rdp_server);
 
 GrdRdpServer *grd_rdp_server_new (GrdContext *context);
 
+void grd_rdp_server_notify_incoming (GSocketService    *service,
+                                     GSocketConnection *connection);
+
 #endif /* GRD_RDP_SERVER_H */
